#!/usr/bin/python2
from PyKDE4 import kdecore,  kdeui
from PyKDE4.kdecore import *
from PyKDE4.kdeui import *
from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *
from PyQt4.QtGui import *
import os, sys

from AkanthophisCommon import *
from AkanthophisUi import *
from PreviewDialog import *
from DownloadMaster import *

class Akanthophis(AkanthophisUi):
  def sequenceChanged(self):
    """When a different sequence is selected, show the appropriate page in the stack"""
    if self.downloadByNumberButton.isChecked():
      self.sequenceStack.setCurrentIndex(self.SEQUENCE_NUMERIC_PAGE)
    elif self.downloadByDateButton.isChecked():
      self.sequenceStack.setCurrentIndex(self.SEQUENCE_DATE_PAGE)
  
  def showPreviewDialog(self):
    """Create a preview of what will be downloaded and to where"""
    downloadInfo = self.getDownloadInfo()
    self.pd = PreviewDialog()
    self.pd.show()
    self.pd.populateTable(downloadInfo)

  def getDownloadInfo(self):
    """Fetch all values from the UI into a dictionary and return it"""
    downloadInfo = {}
    downloadInfo["startDate"] = self.startDateEdit.date()
    downloadInfo["currentDate"] = self.startDateEdit.date()
    downloadInfo["endDate"] = self.endDateEdit.date()
    downloadInfo["formatString"] = self.formatStringEdit.text()
    downloadInfo["startNumber"] = self.startNumberEdit.value()
    downloadInfo["endNumber"] = self.endNumberEdit.value()
    downloadInfo["totalDigits"] = self.totalDigitsEdit.value()
    downloadInfo["redundantZeros"] = self.redundantZerosEdit.value()
    elistqt = self.extensionsListEdit.items()
    elistpy = list()
    for item in elistqt:
      elistpy.append(item)
    if len(elistpy) == 0:
      elistpy.append("{none}")
    downloadInfo["extensionsList"] = elistpy
    if self.downloadByDateButton.isChecked():
      downloadInfo["downloadMethod"] = "byDate"
    elif self.downloadByNumberButton.isChecked():
      downloadInfo["downloadMethod"] = "byNumber"
    downloadInfo["baseUrl"] = self.baseUrlEdit.text()
    downloadInfo["downloadDirectory"] = self.downloadDirectoryEdit.url()
    downloadInfo["overwriteExisting"] = self.overwriteExistingEdit.itemText(self.overwriteExistingEdit.currentIndex())
    downloadInfo["maxConcurrent"] = self.maxConcurrentDownloadsEdit.value()
    downloadInfo["delay"] = self.delayEdit.value()
    downloadInfo["extensionFallback"] =  self.downloadFallbackButton.isChecked()
    daysFilter = []
    if self.dateFilterSundayButton.isChecked():
      daysFilter.append("Sunday")
    if self.dateFilterMondayButton.isChecked():
      daysFilter.append("Monday")
    if self.dateFilterTuesdayButton.isChecked():
      daysFilter.append("Tuesday")
    if self.dateFilterWednesdayButton.isChecked():
      daysFilter.append("Wednesday")
    if self.dateFilterThursdayButton.isChecked():
      daysFilter.append("Thursday")
    if self.dateFilterFridayButton.isChecked():
      daysFilter.append("Friday")
    if self.dateFilterSaturdayButton.isChecked():
      daysFilter.append("Saturday")
    downloadInfo["daysFilter"] = daysFilter
    return downloadInfo
  
  def clearForms(self):
    """Reset all form stuffs to their original values, reset stacks and tab widgets to their original pages, clear tables"""
    self.setInitialValues()
    self.extensionsListEdit.clear()
  
  def downloadFinished(self):
    self.returnButton.setEnabled(True)
    self.cancelButton.setEnabled(False)
    self.setCursor(Qt.ArrowCursor)
    
  def startDownload(self):
    self.rows = dict()
    self.progressMutex = QMutex()
    self.currentRow = 0
    self.mainStack.setCurrentIndex(self.STACK_PROGRESS_PAGE)
    downloadInfo = self.getDownloadInfo()
    self.dm = DownloadMaster(self.addRow, self.updateRow, self.downloadFinished, downloadInfo)
    self.dm.downloadFinishedSignal.connect(self.downloadFinished)
    self.dm.cancelDownloadSignal.connect(self.dm.cancelDownloadHandler)
    self.dm.start()
    self.returnButton.setEnabled(False)
    self.cancelButton.setEnabled(True)
  
  def returnToSetup(self):
    for row in xrange(self.downloadProgressTable.rowCount(), 0, -1):
      self.downloadProgressTable.removeRow(row)
    self.mainStack.setCurrentIndex(self.STACK_SETUP_PAGE)
    
  def cancelDownload(self):
    self.dm.cancelDownloadSignal.emit() 
    
  def showFormatStringHelp(self):
    self.formatHelpDialog = KDialog(self)
    self.formatHelpDialog.setCaption("Date Format Help")
    self.formatHelpDialog.setButtons(KDialog.Ok)
    labelText = "You can intermix just about anything you want here.\n" \
                + "Insert any of the below to create your own custom format.\n" \
          + "You can separate them with anything you want; however, \n" \
          + "I discourage the use of spaces because they get turned into\n" \
          + "'%20', which usually isn't desirable.\n\n\n" \
          + "{d}\tthe day as number without a leading zero (1 to 31)\n" \
          + "{dd}\tthe day as number with a leading zero (01 to 31)\n" \
          + "{ddd}\tthe abbreviated localized day name (e.g. 'Mon' to 'Sun').\n" \
          + "{dddd}\tthe long localized day name (e.g. 'Monday' to 'Sunday').\n" \
          + "{m}\tthe month as number without a leading zero (1 to 12).\n" \
          + "{mm}\tthe month as number with a leading zero (01 to 12).\n" \
          + "{mmm}\tthe abbreviated localized month name (e.g. 'Jan' to 'Dec').\n" \
          + "{mmmm}\tthe long localized month name (e.g. 'January' to 'December').\n" \
          + "{yy}\tthe year as two digit number (00 to 99).\n" \
          + "{yyyy}\tthe year as four digit number.\n\n" \
          + "example: {mm}/{dd}/{yyyy} = 01/22/1970\n"
    label1 = QLabel(labelText, self.formatHelpDialog)
    label1.adjustSize()
    self.formatHelpDialog.setMainWidget(label1)
    self.formatHelpDialog.adjustSize()
    self.formatHelpDialog.show()
  
  def verifyInput(self):
    downloadInfo = self.getDownloadInfo()
    downloadDirectory = downloadInfo["downloadDirectory"]
    baseUrl = downloadInfo["baseUrl"]
    startDate = downloadInfo["startDate"]
    endDate = downloadInfo["endDate"]
    startNumber = downloadInfo["startNumber"]
    endNumber = downloadInfo["endNumber"]
    downloadMethod = downloadInfo["downloadMethod"]
    formatString = downloadInfo["formatString"]
    if os.path.isdir(downloadDirectory) and baseUrl != "":
      if (downloadMethod == "byDate" and startDate.daysTo(endDate) >= 0) and formatString != "" \
      or (downloadMethod == "byNumber" and startNumber <= endNumber):
        self.downloadButton.setEnabled(True)
        self.previewButton.setEnabled(True)
      else:
        self.downloadButton.setEnabled(False)
        self.previewButton.setEnabled(False)
    else:
      self.downloadButton.setEnabled(False)
      self.downloadButton.setEnabled(True)
  
  def addRow(self,localFile,remoteFile):
    """Add a row to the progress table"""
    self.progressMutex.lock()
    localFile = str(localFile)
    remoteFile = str(remoteFile)
    row = self.currentRow
    self.currentRow +=1
    self.rows[remoteFile] = row
    q = QProgressBar()
    q.setMaximum(0)
    q.setMinimum(0)
    self.downloadProgressTable.setRowCount(row+1)
    self.downloadProgressTable.insertRow(row)
    self.downloadProgressTable.setItem(row,3,QTableWidgetItem(localFile))
    self.downloadProgressTable.setItem(row,2,QTableWidgetItem(remoteFile))
    self.downloadProgressTable.setCellWidget(row,1,q)
    self.downloadProgressTable.setItem(row,0,QTableWidgetItem("Starting"))
    self.downloadProgressTable.resizeColumnsToContents()
    self.progressMutex.unlock()
    return(0)
  
  def updateRow(self,remoteFile,progress,maximum,status):
    """Update a row in the progress table"""
    self.progressMutex.lock()
    remoteFile = str(remoteFile)
    progress = int(progress)
    maximum = int(maximum)
    status = str(status)
    row = self.rows[remoteFile]
    self.downloadProgressTable.cellWidget(row,1).setRange(0, maximum)
    self.downloadProgressTable.cellWidget(row,1).setValue(progress)
    self.downloadProgressTable.item(row,0).setText(status)
    self.progressMutex.unlock()
    
  def updateFormatStringPreview(self):
    downloadInfo = self.getDownloadInfo()
    formatString = downloadInfo["formatString"]
    date = downloadInfo["startDate"]
    preview = processFormatString(formatString, date)
    self.formatStringPreview.setText(preview)
    
  def __del__(self):
    self.exiting = True
    print("Akanthophis GUI exiting now")
  
app_name = "akanthophis"
catalog = ""
program_name = ki18n("Akanthophis")
version = "1.1"
description = ki18n("Batch Downloader")
license = KAboutData.License_GPL
copyright = ki18n("(c) 2008-2011 Dustin Widmann")
text = ki18n("none")
home_page = ""
bug_email = "1m.0n.f1r3@gmail.com"

aboutData = KAboutData(app_name, catalog, program_name, version, description, license, copyright, text, home_page, bug_email)

KCmdLineArgs.init(sys.argv, aboutData)

app=KApplication()
akanthophis = Akanthophis()

sys.exit(app.exec_())
