from PyQt4.QtCore import *
from DownloadWorker import *
from AkanthophisCommon import *

class DownloadMaster(QThread):
  downloadFinishedSignal = pyqtSignal()
  cancelDownloadSignal = pyqtSignal()
  def __init__(self, addRow, updateRow, downloadFinished, downloadInfo):
    super(DownloadMaster, self).__init__()
    self.addRow = addRow
    self.updateRow = updateRow
    self.downloadFinished = downloadFinished
    self.downloadInfo = downloadInfo
    self.workers = []
    self.semaphore = QSemaphore(downloadInfo["maxConcurrent"])
    self.currentRow = 0
    self.downloadCancelled = False
    
  def cancelDownloadHandler(self):
    self.downloadCancelled = True
    pass
    
  def run(self):
    sequences = []
    if self.downloadInfo["downloadMethod"] == "byDate":
      sequence = {'sequenceType':'date',\
                  'startDate':self.downloadInfo["startDate"],\
                  'endDate':self.downloadInfo["endDate"],\
                  'daysFilter':self.downloadInfo["daysFilter"], \
                  'formatString':self.downloadInfo["formatString"], \
                  'identifier':'v1sequence'}
    elif self.downloadInfo["downloadMethod"] == "byNumber":
      sequence = {'sequenceType':'numeric', \
                  'startNumber':self.downloadInfo["startNumber"], \
                  'endNumber':self.downloadInfo["endNumber"], \
                  'totalDigits':self.downloadInfo["totalDigits"], \
                  'redundantZeros':self.downloadInfo["redundantZeros"], \
                  'identifier':'v1sequence'}
    sequences.append(sequence)
    extSequence = {'sequenceType':'extension', \
                   'extensionsList':self.downloadInfo["extensionsList"], \
                   'identifier':'extension'}
    sequences.append(extSequence)
    sequenceStringLocal = self.downloadInfo["downloadDirectory"] + "{v1sequence}" + "{extension}"
    sequenceStringRemote = self.downloadInfo["baseUrl"]  + "{v1sequence}" + "{extension}"
    
    self.processSequences(sequenceStringLocal, sequenceStringRemote, sequences)
    
    for worker in self.workers:
      worker.wait()
    self.semaphore.acquire(self.downloadInfo["maxConcurrent"])
    self.downloadFinishedSignal.emit()
    self.exiting = True

  def threadStarter(self, local, remote, extensionsList):
    if not self.downloadCancelled:
      worker = DownloadWorker(local, remote, self.downloadInfo["delay"], self.downloadInfo["overwriteExisting"], self.semaphore, extensionsList)
      worker.updateRowSignal.connect(self.updateRow)
      worker.addRowSignal.connect(self.addRow)
      worker.start()
      self.workers.append(worker)
    
  def processSequences(self, sequenceStringLocal, sequenceStringRemote, sequences):
    if len(sequences) == 1 and sequences[0]['sequenceType'] == 'extension' and self.downloadInfo["extensionFallback"] == True:
      # We're down to just a fallback-style extension sequence now
      self.threadStarter(sequenceStringLocal, sequenceStringRemote, sequences[0]["extensionsList"])
    elif len(sequences) == 1 and sequences[0]['sequenceType'] == 'extension' and self.downloadInfo['extensionFallback'] == False:
      # We're down to just one sequence, and it's not fallback-style
      for sequence in sequences[0]["extensionsList"]: # It should be an extension, lets assume
        local = str(sequenceLocalString).replace('{extension}', sequence)
        remote = str(sequenceRemoteString).replace('{extension}', sequence)
        self.threadStarter(local, remote, False)
    elif len(sequences) > 1:
      # There are multiple sequences remaining to loop through
      currentSequence = sequences[0]
      if currentSequence["sequenceType"] == "numeric":
        startNumber = int(currentSequence["startNumber"])
        endNumber = int(currentSequence["endNumber"])
        totalDigits = int(currentSequence["totalDigits"])
        redundantZeros = int(currentSequence["redundantZeros"])
        currentNumber = startNumber
        redundantZerosString = makeRedundantZerosString(redundantZeros)
        while currentNumber <= endNumber:
          leadingZerosString = makeLeadingZerosString(currentNumber, totalDigits)
          remoteString = str(sequenceStringRemote)
          localString = str(sequenceStringLocal)
          replacement = redundantZerosString + leadingZerosString + str(currentNumber)
          remoteString = remoteString.replace('{%s}' % currentSequence["identifier"], replacement)
          localString = localString.replace('{%s}' % currentSequence["identifier"], replacement)
          currentSequences = []
          for s in sequences:
            currentSequences.append(s)
          
          currentSequences.pop(0)
          self.processSequences(localString, remoteString, currentSequences)
          currentNumber += 1
          
      elif currentSequence["sequenceType"] == "date":
        currentSequence = sequences[0]
        startDate = currentSequence["startDate"]
        endDate = currentSequence["endDate"]
        formatString = currentSequence["formatString"]
        daysFilter = currentSequence["daysFilter"]
        currentDate = QDate(startDate.year(), startDate.month(), startDate.day())
        while currentDate.daysTo(endDate) >  -1:
          dayOfWeek = ''
          if currentDate.dayOfWeek() == 1:
            dayOfWeek = 'Monday'
          elif currentDate.dayOfWeek() == 2:
            dayOfWeek = 'Tuesday'
          elif currentDate.dayOfWeek() == 3:
            dayOfWeek = 'Wednesday'
          elif currentDate.dayOfWeek() == 4:
            dayOfWeek = 'Thursday'
          elif currentDate.dayOfWeek() == 5:
            dayOfWeek = 'Friday'
          elif currentDate.dayOfWeek() == 6:
            dayOfWeek = 'Saturday'
          elif currentDate.dayOfWeek() == 7:
            dayOfWeek = 'Sunday'
          if dayOfWeek in daysFilter:
            dateString = processFormatString(formatString, currentDate)
            remoteString = str(sequenceStringRemote)
            localString = str(sequenceStringLocal)
            remoteString = remoteString.replace("{%s}" % currentSequence["identifier"], dateString)
            localString = localString.replace("{%s}" % currentSequence["identifier"], dateString)
            currentSequences = []
            for s in sequences:
              currentSequences.append(s)
            currentSequences.pop(0)
            self.processSequences(localString, remoteString, currentSequences)
          currentDate = currentDate.addDays(1)
    
