from PyQt4.QtCore import *
from AkanthophisCommon import *
import os, pycurl, random, md5

class DownloadWorker(QThread):
  addRowSignal = pyqtSignal(str,str)
  updateRowSignal = pyqtSignal(str,int,int,str)
  def __init__(self,local,remote,delay,overwriteExisting,semaphore,extensions):
    super(DownloadWorker, self).__init__()
    self.semaphore = semaphore
    self.local = local
    self.remote = remote
    self.extensions = extensions
    self.delay = delay
    self.overwriteExisting = overwriteExisting
    self.workers = []
      
  def run(self):
    if not self.extensions == False and len(self.extensions) > 0:
      #There is an extensions List, fallback style is assumed
      for extension in self.extensions: 
        local = self.local.replace("{extension}", extension)
        remote = self.remote.replace("{extension}", extension)
        self.semaphore.acquire()
        code = self.download(local, remote)
        self.semaphore.release()
        if code == "success" or code =="File Exists":
          # We have a winner here! No need to try the rest.
          break
    else:
      #Just one
      self.download(self.local, self.remote)
      
  def download(self,localPath,remotePath):
    localPath = str(localPath)
    remotePath = str(remotePath)
    
    # Can be Skip, Rename - else it's assumed to be Overwrite
    if os.path.exists(localPath) and self.overwriteExisting == 'Skip':
      # It exists and is to be skipped
      errorMessage = "File Exists"
      wait = self.addRowSignal.emit(localPath,remotePath)
      self.updateRowSignal.emit(remotePath, 0, 1, errorMessage)
      return errorMessage
    if os.path.exists(localPath) and self.overwriteExisting == 'Rename':
      # It exists and we need to rename it, assume the extension begins with a dot
      while True:
        unique = str(md5.md5(str(random.random())).hexdigest())[0:6]
        try:
          localPathStart, localPathFinish = localPath.rsplit('.', 1)
          localPath = localPathStart + "-" + unique + "." + localPathFinish
        except ValueError:
          #There is no dot?! Just append to the end then!
          localPath = localPath + "-" + unique
        if not os.path.exists(localPath):
          break
        
    self.addRowSignal.emit(localPath, remotePath)
    if not os.path.isdir(os.path.dirname(localPath)):
      try:
        os.makedirs(os.path.dirname(localPath))
      except:
        error = "Local Error"
        self.updateRowSignal.emit(remotePath, 0, 1, errorMessage)
        return errorMessage
    
    localFile = open(localPath, 'w')
    self.previousPercent = 0
    
    def bodyCallback(buf):
      localFile.write(buf)
      
    def progressCallback(downloadTotal,downloaded,uploadTotal,uploaded):
      if downloadTotal > 0:
        currentPercent = self.trunc(downloaded/downloadTotal*100)
        if currentPercent > self.previousPercent:
          self.previousPercent = currentPercent
          self.updateRowSignal.emit(remotePath, downloaded,downloadTotal,'downloading')
          
    c = pycurl.Curl()
    c.setopt(c.URL,remotePath)
    c.setopt(c.WRITEFUNCTION, bodyCallback)
    c.setopt(c.NOPROGRESS, 0)
    c.setopt(pycurl.PROGRESSFUNCTION, progressCallback)
    if self.delay > 0:
      self.updateRowSignal.emit(remotePath, 0, 1, "Delaying Start")
      self.msleep(self.delay)
      self.updateRowSignal.emit(remotePath, 0, 1, "Starting")
    c.perform()
    if c.getinfo(pycurl.HTTP_CODE) == 200 and os.path.getsize(localPath) > 0:
      self.updateRowSignal.emit(remotePath, 1, 1, 'finished')
      return "success"
    else:
      self.updateRowSignal.emit(remotePath, 0, 1, str(c.getinfo(pycurl.HTTP_CODE)) + ' error')
      try:
        os.remove(localPath)
      finally:
        return (str(c.getinfo(pycurl.HTTP_CODE)) + ' error')
  def trunc(self, f):
        '''Truncates/pads to nearest integer'''
        i, d = str(f).split('.')
        return int(i)
